#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/fcntl.h>
#include <sys/select.h>
#include <linux/input.h>

typedef unsigned char u8;
#include "einkfb.h"

const double powers[] = { 3.0, 2.0, 1.0, 1.0 / 2.0, 1.0 / 3.0 };
const int depths[] = { 1, 3, 15 };
const int CONTRAST_MAX = (sizeof( powers ) / sizeof( *powers )) - 1;
const int DEPTH_MAX = (sizeof( depths ) / sizeof( *depths )) - 1;

int main()
{
   int depth, contrast, inverted;
   long translate[2][3][5][16];

   int fdFb = open( "/dev/fb0", O_RDWR );
   int fdKeypad = open( "/dev/input/event0", O_RDONLY );
   if( fdFb == -1 || fdKeypad == -1 )
   {
      fprintf( stderr, "Unable to open a device (fds = %d %d)\n", fdFb, fdKeypad );
      return 1;
   }

   for( inverted = 0; inverted < 2; ++inverted )
   {
      for( depth = 0; depth <= DEPTH_MAX; ++depth )
      {
         for( contrast = 0; contrast <= CONTRAST_MAX; ++contrast )
         {
            int shade;
            long* pTable = translate[inverted][depth][contrast];
            for( shade = 0; shade <= 15; ++shade )
            {
               double n = (double)shade / 15.0;
               pTable[shade] = (long)(pow( n, powers[contrast] ) * depths[depth] + 0.5) * (15 / depths[depth]);
               if( inverted )
                  pTable[shade] = 15 - pTable[shade];
            }
         }
      }
   }

   contrast = 2;
   depth = 2;
   inverted = 0;

   while( 1 )
   {
      unsigned char buffer[16];
      if( read( fdKeypad, buffer, 16 ) != 16 )
         break;

      if( buffer[10] == 0x2A && buffer[12] == 1 )
      {
         int fdFiveway = open( "/dev/input/event1", O_RDONLY );
         ioctl( fdFiveway, EVIOCGRAB, 1 );
         fd_set readFds;
         FD_ZERO( &readFds );
         FD_SET( fdKeypad, &readFds );
         FD_SET( fdFiveway, &readFds );
         while( select( fdFiveway + 1, &readFds, NULL, NULL, NULL ) != -1 )
         {
            if( FD_ISSET( fdKeypad, &readFds ) )
            {
               if( read( fdKeypad, buffer, 16 ) != 16 )
                  break;
               if( buffer[10] == 0x2A && buffer[12] == 0 )
                  break;
            }
            if( FD_ISSET( fdFiveway, &readFds ) )
            {
               if( read( fdFiveway, buffer, 16 ) != 16 )
                  break;
               if( buffer[12] == 1 )
               {
                  int update = 0;
                  if( buffer[10] == 0x7A && contrast < CONTRAST_MAX )
                     update = 1, ++contrast;
                  else if( buffer[10] == 0x6A && depth < DEPTH_MAX )
                     update = 1, ++depth;
                  else if( buffer[10] == 0x7B && contrast > 0 )
                     update = 1, --contrast;
                  else if( buffer[10] == 0x69 && depth > 0 )
                     update = 1, --depth;
                  else if( buffer[10] == 0x5C )
                     update = 1, inverted ^= 1;

                  if( update )
                  {
                     ioctl( fdFb, FBIO_EINK_SET_GAMMA_RAMP, translate[inverted][depth][contrast] );
                     ioctl( fdFb, FBIO_EINK_UPDATE_DISPLAY, 0 );
                  }
               }
            }
            FD_SET( fdKeypad, &readFds );
            FD_SET( fdFiveway, &readFds );
         }
         ioctl( fdFiveway, EVIOCGRAB, 0 );
         close( fdFiveway );
      }
   }

   close( fdFb );
   close( fdKeypad );

   return 0;
}

